- CUDA's dynamic parallelism extension allows device-side kernel
launches, which share the identical syntax to host-side launches, e.g.,
kernel<<<Dg, Db, Ns, S>>>(arguments);
but differ from the code generation. That device-side kernel launches is
eventually translated into the following sequence
config = cudaGetParameterBuffer(alignment, size);
// setup arguments by copying them into `config`.
cudaLaunchDevice(func, config, Dg, Db, Ns, S);
- To support the device-side kernel launch, 'CUDAKernelCallExpr' is
reused but its config expr is set to a call to 'cudaLaunchDevice'.
During the code generation, 'CUDAKernelCallExpr' is expanded into the
sequence aforementioned.
- As the device-side kernel launch requires the source to be compiled as
relocatable device code and linked with '-lcudadevrt'. Linkers are
changed to pass relevant link options to 'nvlink'.
35 lines
1.3 KiB
Plaintext
35 lines
1.3 KiB
Plaintext
// RUN: %clang_cc1 -fsyntax-only -verify %s
|
|
// RUN: %clang_cc1 -fsyntax-only -fcuda-is-device -verify=dev,expected %s
|
|
|
|
#include "Inputs/cuda.h"
|
|
|
|
__host__ void h1h(void);
|
|
__device__ void h1d(void); // expected-note {{candidate function not viable: call to __device__ function from __host__ function}}
|
|
__host__ __device__ void h1hd(void);
|
|
__global__ void h1g(void);
|
|
|
|
struct h1ds { // expected-note {{requires 1 argument}}
|
|
// expected-note@-1 {{candidate constructor (the implicit move constructor) not viable}}
|
|
__device__ h1ds(); // expected-note {{candidate constructor not viable: call to __device__ function from __host__ function}}
|
|
};
|
|
|
|
__host__ void h1(void) {
|
|
h1h();
|
|
h1d(); // expected-error {{no matching function}}
|
|
h1hd();
|
|
h1g<<<1, 1>>>();
|
|
h1ds x; // expected-error {{no matching constructor}}
|
|
}
|
|
|
|
__host__ void d1h(void); // expected-note {{candidate function not viable: call to __host__ function from __device__ function}}
|
|
__device__ void d1d(void);
|
|
__host__ __device__ void d1hd(void);
|
|
__global__ void d1g(void);
|
|
|
|
__device__ void d1(void) {
|
|
d1h(); // expected-error {{no matching function}}
|
|
d1d();
|
|
d1hd();
|
|
d1g<<<1, 1>>>(); // expected-error {{kernel launch from __device__ or __global__ function requires relocatable device code (i.e. requires -fgpu-rdc)}}
|
|
}
|